 const sample = [{
     language: 'Rust',
     value: 78.9,
     color: '#000000'
   },
   {
     language: 'Kotlin',
     value: 75.1,
     color: '#00a2ee'
   },
   {
     language: 'Python',
     value: 68.0,
     color: '#fbcb39'
   },
   {
     language: 'TypeScript',
     value: 67.0,
     color: '#007bc8'
   },
   {
     language: 'Go',
     value: 65.6,
     color: '#65cedb'
   },
   {
     language: 'Swift',
     value: 65.1,
     color: '#ff6e52'
   },
   {
     language: 'JavaScript',
     value: 61.9,
     color: '#f9de3f'
   },
   {
     language: 'C#',
     value: 60.4,
     color: '#5d2f8e'
   },
   {
     language: 'F#',
     value: 59.6,
     color: '#008fc9'
   },
   {
     language: 'Clojure',
     value: 59.6,
     color: '#507dca'
   }
 ];

 const svg = d3.select('svg');
 const svgContainer = d3.select('#container');

 const width = +svg.attr('width');
 const height = +svg.attr('height');

 const xValue = (s) => s.language;
 const yValue = (s) => s.value;
 const margin = {
   top: 75,
   right: 20,
   bottom: 20,
   left: 50
 };
 const innerWidth = width - margin.left - margin.right;
 const innerHeight = height - margin.top - margin.bottom;

 const chart = svg.append('g')
   .attr('transform', `translate(${margin.left}, ${margin.top})`);
  

 const xScale = d3.scaleBand()
   .range([0, innerWidth])
   .domain(sample.map(xValue))
   .padding(0.4)

 const yScale = d3.scaleLinear()
   .range([innerHeight, 0])
   .domain([0, 100]);
   
/* 	const dragBarWidth = 20;
	 const dragTop = d3.drag()
	    .origin(Object)
	    .on("drag", tdragresize);  
	 const dragbartop = chart.append("rect")
	   .attr("x", function(d) { return d.x + (dragbarw/2); })
	   .attr("y", function(d) { return d.y - (dragbarw/2); })
	   .attr("height", dragBarWidth)
	   .attr("id", "dragleft")
	   .attr("width", xScale.bandwidth() + dragBarWidth)
	   .attr("fill", "lightgreen")
	   .attr("fill-opacity", 0.5)
	   .attr("cursor", "ns-resize")
	   .call(dragTop); */

 // vertical grid lines
 // const makeXLines = () => d3.axisBottom()
 //   .scale(xScale)

 chart.append('g')
   .attr('transform', `translate(0, ${innerHeight})`)
   .call(d3.axisBottom(xScale));

 chart.append('g')
   .call(d3.axisLeft(yScale));

 // vertical grid lines
 // chart.append('g')
 //   .attr('class', 'grid')
 //   .attr('transform', `translate(0, ${height})`)
 //   .call(makeXLines()
 //     .tickSize(-height, 0, 0)
 //     .tickFormat('')
 //   )

 	chart.append('g')
   .attr('class', 'grid')
   .call(d3.axisLeft(yScale)
     .tickSize(-innerWidth, 0, 0)
     .tickFormat('')
   )

 	const barGroups = chart.selectAll()
   .data(sample)
   .enter()
   .append('g')

 	barGroups
   .append('rect')
   .attr('class', 'bar')
   .attr('x', (g) => xScale(xValue(g)))
   .attr('y', (g) => yScale(yValue(g)))
   .attr('height', (g) => innerHeight - yScale(yValue(g)))
   .attr('width', xScale.bandwidth())
   .on('mouseenter', barSelect)
   .on('mouseleave', barDeSelect);
   

	function barSelect(actual, i) {
    d3.selectAll('.value')
      .attr('opacity', 0)

    d3.select(this)
      .transition()
      .duration(300)
      .attr('opacity', 0.6)
      .attr('x', (a) => xScale(xValue(a)) - 5)
      .attr('width', xScale.bandwidth() + 10)


    const y = yScale(yValue(actual))

    line = chart.append('line')
      .attr('id', 'limit')
      .attr('x1', 0)
      .attr('y1', y)
      .attr('x2', innerWidth)
      .attr('y2', y)

    barGroups.append('text')
      .attr('class', 'divergence')
      .attr('x', (a) => xScale(xValue(a)) + xScale.bandwidth() / 2)
      .attr('y', (a) => yScale(yValue(a)) + 30)
      .attr('fill', 'white')
      .attr('text-anchor', 'middle')
      .text((a, idx) => {
      const divergence = (yValue(a) - yValue(actual)).toFixed(1)


      let text = ''
      if (divergence > 0) text += '+'
      text += `${divergence}%`

      return idx !== i ? text : '';
    })
  }
  
  function barDeSelect() {
     d3.selectAll('.value')
       .attr('opacity', 1)

     d3.select(this)
       .transition()
       .duration(300)
       .attr('opacity', 1)
       .attr('x', (a) => xScale(xValue(a)))
       .attr('width', xScale.bandwidth())

     chart.selectAll('#limit').remove()
     chart.selectAll('.divergence').remove()
   }
   
/*    function tdragresize(d) {
     var oldy = d.y; 
     //Max x on the right is x + width - dragbarw
     //Max x on the left is 0 - (dragbarw/2)
     barHeight = yScale(yValue(d))
     d.y = Math.max(0, Math.min(d.y + barHeight - (dragBarWidth / 2), d3.event.y)); 
     barHeight = barHeight + (oldy - d.y);
     dragbartop
       .attr("y", function(d) { return d.y - (dragBarWidth / 2); });
   
     dragrect
       .attr("y", function(d) { return d.y; })
       .attr("height", barHeight);
   
     dragbarleft 
       .attr("y", function(d) { return d.y + (dragBarWidth/2); })
       .attr("height", barHeight - dragBarWidth);
     dragbarright 
       .attr("y", function(d) { return d.y + (dragBarWidth/2); })
       .attr("height", barHeight - dragBarWidth);
   } */

  const dragBarGroups = barGroups
    .append('rect')
    .attr('x', (a) => xScale(xValue(a)) - 5)
    .attr('y', (a) => yScale(yValue(a)))
    .attr('height', 5)
    .attr('width', xScale.bandwidth() + 10) 	
    .attr('opacity', 0.5)
    .attr('fill', 'blue')
    .call(d3.drag()
        .on("start", dragstarted)
        .on("drag", dragged)
        .on("end", dragended));

  // Function to handle what happens when drag is started
  function dragstarted(d) {
    d3.select(this).raise().classed("active", true);
  }

  // Function to handle what happens when drag is in progress
  function dragged(d) {
    d3.select(this).attr("y", d.y = d3.event.y);
  }

  // Function to handle what happens when drag is ended
  function dragended(d) {
    d3.select(this).classed("active", false);
		dragBarGroups
    	.attr("y", function(d) { return d.y; })
  /*     dragBarGroups.
      attr('y', this.y) */
  }

	function newValue(d) {
  	
  }

 barGroups
   .append('text')
   .attr('class', 'value')
   .attr('x', (a) => xScale(xValue(a)) + xScale.bandwidth() / 2)
   .attr('y', (a) => yScale(yValue(a)) + 30)
   .attr('text-anchor', 'middle')
   .text((a) => `${yValue(a)}%`)

 chart
   .append('text')
   .attr('class', 'label')
   .attr('x', -innerHeight / 2)
   .attr('y', -30)
   .attr('transform', 'rotate(-90)')
   .attr('text-anchor', 'middle')
   .text('Love meter (%)')

 chart.append('text')
   .attr('class', 'label')
   .attr('x', innerWidth / 2)
   .attr('y', innerHeight + 40)
   .attr('text-anchor', 'middle')
   .text('Languages')

 chart.append('text')
   .attr('class', 'title')
   .attr('x', innerWidth / 2)
   .attr('y', -20)
   .attr('text-anchor', 'middle')
   .text('Most loved programming languages in 2018')

 chart.append('text')
   .attr('class', 'source')
   .attr('x', innerWidth - 100)
   .attr('y', innerHeight + 50)
   .attr('text-anchor', 'start')
   .text('Source: Stack Overflow, 2018')
